@@ -76,6 +76,10 @@ static const BuiltinColourMatrix s_matrices[] = {
       15,
       0,
       { 7610, -2780, -576, -4614, 12195, 2733, -1375, 2393, 6490 } },
+    { OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF8),
+      15,
+      0,
+      { 7610, -2780, -576, -4614, 12195, 2733, -1375, 2393, 6490 } },
     { OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF10),
       15,
       0,
@@ -356,6 +360,10 @@ static const BuiltinColourMatrix s_matrices[] = {
       0,
       0,
       { 9744, -3905, -779, -4899, 12807, 2324, -798, 1630, 5827 } },
+    { OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_DC_S5M2),
+      0,
+      0,
+      { 10308, -4206, -783, -4088, 12102, 2229, -125, 1051, 5912 } },
     { OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_ZS70),
       0,
       0,
@@ -433,6 +441,7 @@ const IfdFile::camera_ids_t Rw2File::s_def[] = {
     { "DMC-GF5", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF5) },
     { "DMC-GF6", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF6) },
     { "DMC-GF7", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF7) },
+    { "DMC-GF8", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF8) },
     { "DC-GF10", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GF10) },
     { "DMC-GX1", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GX1) },
     { "DMC-GX7", OR_MAKE_PANASONIC_TYPEID(OR_TYPEID_PANASONIC_GX7) },
